import matplotlib.pyplot as plt
import matplotlib.cm as cm

import stregion
try:
    from astropy.io import fits as pyfits
except ImportError:
    from astropy.io import fits as pyfits

# read in the image
xray_name="pspc_skyview.fits"
f_xray = pyfits.open(xray_name)

try:
    import pywcsgrid2
    ax=pywcsgrid2.subplot(111, header=f_xray[0].header)
except ImportError:
    ax=plt.subplot(111)

ax.imshow(f_xray[0].data, cmap=cm.gray, vmin=0., vmax=0.00038, origin="lower")

reg_name = "test.reg"
r = stregion.open(reg_name).as_imagecoord(header=f_xray[0].header)

from stregion.mpl_helper import properties_func_default

# Use custom function for patch attribute
def fixed_color(shape, saved_attrs):

    attr_list, attr_dict = saved_attrs
    attr_dict["color"] = "red"
    kwargs = properties_func_default(shape, (attr_list, attr_dict))

    return kwargs

# select region shape with tag=="Group 1"
r1 = stregion.ShapeList([rr for rr in r if rr.attr[1].get("tag") == "Group 1"])
patch_list1, artist_list1 = r1.get_mpl_patches_texts(fixed_color)

r2 = stregion.ShapeList([rr for rr in r if rr.attr[1].get("tag") != "Group 1"])
patch_list2, artist_list2 = r2.get_mpl_patches_texts()

for p in patch_list1 + patch_list2:
    ax.add_patch(p)
for t in artist_list1 + artist_list2:
    ax.add_artist(t)

plt.show()




from stregion import read_region, read_region_as_imagecoord

import math

try:
    from astropy.io import fits as pyfits
except ImportError:
    from astropy.io import fits as pyfits

from demo_helper import pyfits_card_fromstring

def test_header():
    cards = pyfits.CardList()
    for l in open("test.header"):
        card = pyfits_card_fromstring(l.strip())
        cards.append(card)
    h = pyfits.Header(cards)
    return h

def print_region(r):
    for i, l in enumerate(r):
        print("[region %d]" % (i+1))
        print()
        print("%s; %s(%s)" % (l.coord_format,
                              l.name,
                              ", ".join([str(s) for s in l.coord_list])))

        print(l.attr[0])
        print(", ".join(["%s=%s" % (k, v.strip()) for k, v in list(l.attr[1].items())]))
        print()

if __name__ == "__main__":
    print("** coordinate in FK5 **")
    print()
    region_name = "test01_print.reg"
    #region_name = "test_text.reg"
    #region_name = "test01.reg"
    r = read_region(open(region_name).read())
    print_region(r)

    print()
    print()
    print("** coordinate in image **")
    print()
    header = test_header()
    r = read_region_as_imagecoord(open(region_name).read(), header)
    print_region(r)




#!/usr/bin/env python
import fileinput
import glob
import os
import shutil
import sys

def check_build():
    build_dirs = ['build', 'build/doctrees', 'build/html',
                  '_static', '_templates']
    for d in build_dirs:
        try:
            os.mkdir(d)
        except OSError:
            pass

def html():
    check_build()
    os.system('python `which sphinx-build` -b html -d build/doctrees . build/html')


def clean():
    shutil.rmtree('build')

def all():
    #figs()
    html()


funcd = {'html':html,
         'clean':clean,
         'all':all,
         }


if len(sys.argv)>1:
    for arg in sys.argv[1:]:
        func = funcd.get(arg)
        if func is None:
            raise SystemExit('Do not know how to handle %s; valid args are'%(
                    arg, funcd.keys()))
        func()
else:
    all()



import matplotlib.pyplot as plt
import stregion

import math

try:
    from astropy.io import fits as pyfits
except ImportError:
    from astropy.io import fits as pyfits

# At some point, pyfits.Card.fromstring has changed from unbound
# method to bounded method.

if pyfits.Card.fromstring.__self__: # 
    def pyfits_card_fromstring(l):
        return pyfits.Card.fromstring(l)
else:
    def pyfits_card_fromstring(l):
        c = pyfits.Card()
        return c.fromstring(l)

def demo_header():
    cards = pyfits.CardList()
    for l in open("sample_fits01.header"):
        card = pyfits_card_fromstring(l.strip())
        cards.append(card)
    h = pyfits.Header(cards)
    return h


def show_region(fig, region_list):
    h = demo_header()

    n = len(region_list)
    nx = int(math.ceil(n**.5))
    ny = int(math.ceil(1.*n/nx))


    nrows_ncols = (ny, nx)

    grid = [plt.subplot(ny, nx, i+1) for i in range(n)]

    for ax, reg_name in zip(grid, region_list):
        ax.set_aspect(1)

        r = stregion.open(reg_name).as_imagecoord(h)

        patch_list, text_list = r.get_mpl_patches_texts()
        for p in patch_list:
            ax.add_patch(p)
        for t in text_list:
            ax.add_artist(t)

        if plt.rcParams["text.usetex"]:
            reg_name = reg_name.replace("_", r"\_")
        ax.set_title(reg_name, size=10)
        for t in ax.get_xticklabels() + ax.get_yticklabels():
            t.set_visible(False)

    return grid




try:
    from astropy.io import fits as pyfits
except ImportError:
    from astropy.io import fits as pyfits

from demo_helper import pyfits_card_fromstring

import matplotlib.pyplot as plt

import stregion

# read in the image
def demo_header():
    cards = pyfits.CardList()
    for l in open("sample_fits02.header"):
        card = pyfits_card_fromstring(l.strip())
        cards.append(card)
    h = pyfits.Header(cards)
    return h

header = demo_header() # sample fits header
shape = (header["NAXIS1"], header["NAXIS2"])

reg_name = "test.reg"
r = stregion.open(reg_name).as_imagecoord(header)
m = r.get_mask(shape=shape)


fig = plt.figure(1, figsize=(7,5))
ax = plt.subplot(121)
plt.imshow(m, origin="lower")

patch_list, text_list = r.get_mpl_patches_texts()
for p in patch_list:
    ax.add_patch(p)
for t in text_list:
    ax.add_artist(t)

# another region

reg_name = "test02.reg"
r = stregion.open(reg_name).as_imagecoord(header)
m = r.get_mask(shape=shape)

ax = plt.subplot(122)
plt.imshow(m, origin="lower")

patch_list, text_list = r.get_mpl_patches_texts()
for p in patch_list:
    ax.add_patch(p)
for t in text_list:
    ax.add_artist(t)

plt.show()




from .wcs_helper import estimate_cdelt, estimate_angle
from .region_numbers import CoordOdd, CoordEven, Distance, Angle
from .parser_helper import Shape, CoordCommand
from .region_numbers import SimpleNumber, SimpleInteger

import copy


def convert_to_imagecoord(cl, fl, wcs_proj, sky_to_sky, xy0, rot_wrt_axis=1):
    fl0 = fl
    new_cl = []

    while cl:
        if len(fl) == 0:
            fl = fl0

        if fl[0] == CoordOdd and fl[1] == CoordEven:

            ll = sky_to_sky([cl[0]], [cl[1]])
            x0, y0 = wcs_proj.topixel(ll)
            xy0 = [x0[0], y0[0]]
            new_cl.extend(xy0)
            cl = cl[2:]
            fl = fl[2:]
        elif fl[0] == Distance:
            degree_per_pixel = estimate_cdelt(wcs_proj, *xy0)
            new_cl.append(cl[0]/degree_per_pixel)
            cl = cl[1:]
            fl = fl[1:]
        elif fl[0] == Angle:
            rot1, rot2 = estimate_angle(wcs_proj, xy0[0], xy0[1], sky_to_sky)
            if rot_wrt_axis == 1:
                new_cl.append(cl[0]+rot1-180.)
            else:
                new_cl.append(cl[0]+rot2-90.) # use the angle between the Y axis and North

            cl = cl[1:]
            fl = fl[1:]
        else:
            new_cl.append(cl[0])
            cl = cl[1:]
            fl = fl[1:]

    return new_cl, xy0


def convert_physical_to_imagecoord(cl, fl, pc):
    fl0 = fl
    new_cl = []

    while cl:
        if len(fl) == 0:
            fl = fl0

        if fl[0] == CoordOdd and fl[1] == CoordEven:

            xy0 = pc.to_image(cl[0], cl[1])
            new_cl.extend(xy0)
            cl = cl[2:]
            fl = fl[2:]
        elif fl[0] == Distance:
            new_cl.append(pc.to_image_distance(cl[0]))
            cl = cl[1:]
            fl = fl[1:]
        else:
            new_cl.append(cl[0])
            cl = cl[1:]
            fl = fl[1:]

    return new_cl




def check_wcs_and_convert(args, all_dms=False):

    is_wcs = False

    value_list = []
    for a in args:
        if isinstance(a, SimpleNumber) or isinstance(a, SimpleInteger) \
               or all_dms:
            value_list.append(a.v)
        else:
            value_list.append(a.degree)
            is_wcs = True

    return is_wcs, value_list


def check_wcs(l):
    default_coord = "physical"

    for l1, c1 in l:
        if isinstance(l1, CoordCommand):
            default_coord = l1.text.lower()
            continue
        if isinstance(l1, Shape):
            if default_coord == "galactic":
                is_wcs, coord_list = check_wcs_and_convert(l1.params,
                                                           all_dms=True)
            else:
                is_wcs, coord_list = check_wcs_and_convert(l1.params)

            if is_wcs and (default_coord == "physical"): # ciao format
                coord_format = "fk5"
            else:
                coord_format = default_coord

            l1n = copy.copy(l1)

            l1n.coord_list = coord_list
            l1n.coord_format = coord_format

            yield l1n, c1
        else:
            yield l1, c1






import os
import sys
import warnings

from setuptools import setup, Extension
from Cython.Build import cythonize


try:
    import numpy
except ImportError:
    warnings.warn(
        "numpy must be installed to build the filtering module.")
    sys.exit(1)

try:
    numpy_include = numpy.get_include()
except AttributeError:
    numpy_include = numpy.get_numpy_include()

CYTHON_SOURCE_DIR = "src"
CYTHON_SOURCE_FILES = ["_region_filter.pyx"]
EXTRA_COMPILE_ARGS = [
    "-Wall",
    "-Wextra",
    "-std=gnu99",
]

extensions = [
    Extension(
        "stregion._region_filter",
        [os.path.join(CYTHON_SOURCE_DIR, x) for x in CYTHON_SOURCE_FILES],
        include_dirs=[
            CYTHON_SOURCE_DIR,
            numpy_include,
        ],
        extra_compile_args=EXTRA_COMPILE_ARGS,
    ),
]

setup(ext_modules=cythonize(extensions))



"""
:Description: Sphinx extension to remove leading under-scores from directories names in the html build output directory.
"""
import os
import shutil

from docutils.io import StringOutput
from sphinx.util import relative_uri

from sphinx.builders.html import StandaloneHTMLBuilder as RealStandaloneHTMLBuilder

class StandaloneHTMLBuilder(RealStandaloneHTMLBuilder):
    def write_doc(self, docname, doctree):
        destination = StringOutput(encoding='utf-8')
        doctree.settings = self.docsettings

        self.secnumbers = self.env.toc_secnumbers.get(docname, {})
        self.imgpath = relative_uri(self.get_target_uri(docname), 'images')
        self.post_process_images(doctree)
        self.dlpath = relative_uri(self.get_target_uri(docname), 'downloads')
        self.docwriter.write(doctree, destination)
        self.docwriter.assemble_parts()
        body = self.docwriter.parts['fragment']
        metatags = self.docwriter.clean_meta

        ctx = self.get_doc_context(docname, body, metatags)
        self.index_page(docname, doctree, ctx.get('title', ''))
        self.handle_page(docname, ctx, event_arg=doctree)



def setup(app):
    """
    Add a html-page-context  and a build-finished event handlers
    """
    app.connect('html-page-context', change_pathto)
    app.connect('build-finished', move_private_folders)

    import sphinx.builders.html
    sphinx.builders.html.StandaloneHTMLBuilder = StandaloneHTMLBuilder
    
def change_pathto(app, pagename, templatename, context, doctree):
    """
    Replace pathto helper to change paths to folders with a leading underscore.
    """
    pathto = context.get('pathto')
    def gh_pathto(otheruri, *args, **kw):
        if otheruri.startswith('_'):
            otheruri = otheruri[1:]
        return pathto(otheruri, *args, **kw)
    context['pathto'] = gh_pathto
    
def move_private_folders(app, e):
    """
    remove leading underscore from folders in in the output folder.
    
    :todo: should only affect html built
    """
    def join(dir):
        return os.path.join(app.builder.outdir, dir)
    
    for item in os.listdir(app.builder.outdir):
        if item.startswith('_') and os.path.isdir(join(item)):
            if os.path.exists(join(item[1:])):
                shutil.rmtree(join(item[1:]))
            shutil.move(join(item), join(item[1:]))



from __future__ import absolute_import


import numpy as np
#from pylab import savefig
import stregion

try:
    from astropy import wcs as pywcs
except ImportError:
    from astropy import wcs as pywcs

def test_estimate_cdelt():
    l,b=0,0

    # This is the ra,dec of l,b=0,0
    ra,dec=266.404497776,-28.9364329295

    # This is 'almost' the ra,dec of l,b=0,0 - works
    # ra,dec=266.40,-28.93

    wcs = pywcs.WCS(naxis=2)

    wcs.wcs.crpix = [5.5, 5.5]
    wcs.wcs.cdelt = [0.1, -0.1]
    wcs.wcs.crval = [l, b]
    wcs.wcs.ctype = ["GLON-ZEA".encode("ascii"), "GLAT-ZEA".encode("ascii")]

    import stregion.wcs_helper as wcs_helper
    proj = wcs_helper.get_kapteyn_projection(wcs)
    cdelt = wcs_helper.estimate_cdelt(proj, 5.5, 5.5)
    
    assert np.allclose([cdelt], [0.1])

    region_string="fk5; circle(%s, %s, 0.5000)" % (ra,dec)
    reg = stregion.parse(region_string).as_imagecoord(wcs)

    assert np.allclose([reg[0].coord_list[-1]], [0.5/0.1])
